<?php
namespace Ep\Gath\Adapter;

use Ep\Gath\Exception;
use GuzzleHttp\ClientInterface as HttpClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Cache\CacheItemPoolInterface;
use Zend\Log\LoggerInterface;

/**
 * Adapter za novi REST API.
 *
 * @package Ep\Gath\Adapter
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class RestAdapter implements AdapterInterface
{
	/** @var HttpClientInterface */
	private $client;
	/** @var CacheItemPoolInterface */
	private $cache;
	/** @var LoggerInterface */
	private $log;
	/** @var integer */
	private $foundRows;

	/**
	 * RestAdapter constructor.
	 *
	 * @param HttpClientInterface $client
	 * @param CacheItemPoolInterface $cache
	 * @param LoggerInterface $log
	 */
	public function __construct(HttpClientInterface $client, CacheItemPoolInterface $cache, LoggerInterface $log)
	{
		$this->client = $client;
		$this->cache = $cache;
		$this->log = $log;
	}

	/**
	 * Vraća broj redova koje bi baza vratila da nismo ograničili listu.
	 *
	 * @return int
	 */
	public function getFoundRows()
	{
		return intval($this->foundRows);
	}

    /**
     * Vrši request prema Gathu.
     *
     * @param $path
     * @param array $params
     *
     * @return array
     *
     * @throws GuzzleException
     */
	protected function request($path, array $params)
	{
		try {
			$response = $this->client->request('POST', $path, [
				'json' => $params,
			]);

			if (300 <= $response->getStatusCode()) {
				$this->log->err(sprintf('Došlo je do greške z Gathom za: %s %s', $path, json_encode($params)));
				throw new \RuntimeException('Gath greška: ' . $response->getReasonPhrase());
			}

			$recordCount = $response->getHeader('X-RecordCount');
			$this->foundRows = intval(array_shift($recordCount));

			$jsonContent = (string) $response->getBody();
			$data = json_decode($jsonContent, true);

			if (null === $data) {
				$this->log->err(sprintf(
					"Greška prilikom dekodiranja JSON-a: %s.\nJSON: %s",
					json_last_error_msg(),
					$jsonContent
				));
				throw new Exception\RuntimeException('Greška prilikom dekodiranja JSON-a.');
			}

			if (!isset($data['Results'])) {
				throw new Exception\RuntimeException("Objekt iz Gatha ne sadrži polje 'Results'.");
			}

			if (!is_array($data['Results'])) {
				throw new Exception\RuntimeException(sprintf(
					"Polje 'Results' mora sadržavati array, a ne %s",
					is_object($data) ? get_class($data['Results']) : gettype($data['Results'])
				));
			}

			return $data['Results'];
		} catch (\Exception $e) {
			$this->log->err(sprintf('Došlo je do greške z Gathom za: %s %s', $path, json_encode($params)));
			throw new Exception\RuntimeException('Ne možemo se spojiti na Gath!', 0, $e);
		}
	}

    /**
     * Vraća cijene za date proizvode.
     *
     * @param array $skus
     * @param bool $retail
     * @param bool $wholesale
     * @param bool $discounts
     * @param string|null $username
     *
     * @return mixed
     *
     * @throws GuzzleException
     */
	public function getPrices(array $skus, bool $retail = true, bool $wholesale = false, bool $discounts = true, string $username = null)
	{
		return $this->request('/prices/by-codes', [
			'Username' => $username,
			'Codes' => $skus,
			'PriceRabatePolicy' => $wholesale,
			'Discount' => $discounts,
		]);
	}

    /**
     * Vraća stanje zalihe za date proizvode.
     *
     * @param array $skus
     * @param string|null $username
     *
     * @return mixed
     *
     * @throws GuzzleException
     */
	public function getStock(array $skus, string $username = null)
	{
		return $this->request('/stock/by-codes', [
			'Username' => $username,
			'Codes' => $skus,
		]);
	}

    /**
     * Vraća proizvode iz date kategorije.
     *
     * @param int $categoryId
     * @param int $order
     * @param int $limit
     * @param int $offset
     *
     * @return mixed
     *
     * @throws GuzzleException
     */
	public function getProductsByCategory(int $categoryId, int $order, int $limit = 10, int $offset = 0)
	{
		return $this->request('/product/bycategory', [
			'CategoryId' => $categoryId,
			'OrderBy' => $order,
			'Limit' => $limit,
			'Offset' => $offset,
            'SortNoStockLast' => true,
		]);
	}


    /**
     * Vraća proizvode po id-u.
     *
     * @param array $productIds
     *
     * @return mixed
     *
     * @throws GuzzleException
     */
	public function getProductsByIds(array $productIds)
	{
		return $this->request('/product/byids', [
			'Ids' => $productIds,
		]);
	}

    /**
     * @param array $tecDocPairs
     * @param int $order
     * @param int $limit
     * @param int $offset
     *
     * @return mixed
     *
     * @throws GuzzleException
     */
	public function getTecDocProducts(array $tecDocPairs, int $order, int $limit = 10, int $offset = 0)
	{
		return $this->request('/product/bytecdocids', [
			'TecDocKeys' => $tecDocPairs,
			'OrderBy' => $order,
			'Limit' => $limit,
			'Offset' => $offset,
            'SortNoStockLast' => true,
		]);
	}

    /**
     * @param $motoId
     * @param $genIds
     * @param int $order
     * @param int $limit
     * @param int $offset
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getMotoProducts($motoId, $genIds, int $order, int $limit = 10, int $offset = 0)
    {
        return $this->request('/moto/articles', [
            'MotoId' => $motoId,
            'GenArtIds' => $genIds,
            'OrderBy' => $order,
            'Limit' => $limit,
            'Offset' => $offset,
            'SortNoStockLast' => true,
        ]);
    }
}