<?php
namespace Ep\Gath\Hydrator;

use Ep\Gath\Entity\ProductBrand;
use Ep\Gath\Entity\ProductImage;
use Ep\Gath\Entity\ProductShippingClass;
use Interop\Container\ContainerInterface;
use Interop\Container\Exception\ContainerException;
use Zend\Hydrator\HydratorInterface;
use Zend\Hydrator\HydratorPluginManager;
use Zend\Hydrator\NamingStrategy\MapNamingStrategy;
use Zend\Hydrator\Reflection as ReflectionHydrator;
use Zend\Hydrator\Strategy\ClosureStrategy;
use Zend\Hydrator\Strategy\DateTimeFormatterStrategy;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Exception\ServiceNotFoundException;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Class ProductHydratorFactory
 *
 * @package Ep\Gath\Hydrator
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ProductHydratorFactory implements FactoryInterface
{
	/**
	 * Create an object
	 *
	 * @param  ContainerInterface $container
	 * @param  string $requestedName
	 * @param  null|array $options
	 *
	 * @return object
	 * @throws ServiceNotFoundException if unable to resolve the service.
	 * @throws ServiceNotCreatedException if an exception is raised when
	 *     creating a service.
	 * @throws ContainerException if any other error occurs
	 */
	public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
	{
		/** @var HydratorPluginManager $hydrators */
		$hydrators = $container->get('HydratorManager');
		/** @var HydratorInterface $brandHydrator */
		$brandHydrator = $hydrators->get(ProductBrand::class);
		/** @var HydratorInterface $shippingClassHydrator */
		$shippingClassHydrator = $hydrators->get(ProductShippingClass::class);
		/** @var HydratorInterface $imageHydrator */
		$imageHydrator = $hydrators->get(ProductImage::class);

		$hydrator = new ReflectionHydrator();

		$hydrator->setNamingStrategy(new MapNamingStrategy([
			'id' => 'id',
			'sku' => 'sku',
			'name' => 'name',
			'description' => 'description',
			'short_description' => 'shortDescription',
			'date_modified' => 'dateModified',
			'tax_classHR' => 'fallbackTaxClass',
			'tax_class' => 'taxClass',
			'featured' => 'featured',
			'wsbrandno' => 'tecdocBrandNo',
			'wsartno' => 'tecdocArticleNo',
			'Brands' => 'brands',
			'ShippingClass' => 'shippingClasses',
			'Images' => 'images',
            'minimumpurchasequantity' => 'minimumPurchaseQuantity',
		]));

		$hydrator->addStrategy('dateModified', new DateTimeFormatterStrategy('Y-m-d H:i:s.u'));

		$hydrator->addStrategy('brands', new ClosureStrategy(
			function() { return []; },
			function($brands) use ($brandHydrator) {
				$hydrated = [];

				if (is_array($brands)) {
					foreach ($brands as $brand) {
						$hydrated[] = $brandHydrator->hydrate($brand, new ProductBrand());
					}
				}

				return $hydrated;
			}
		));

		$hydrator->addStrategy('shippingClasses', new ClosureStrategy(
			function() { return []; },
			function($shippingClasses) use ($shippingClassHydrator) {
				$hydrated = [];

				if (is_array($shippingClasses)) {
					foreach ($shippingClasses as $shippingClass) {
						$hydrated[] = $shippingClassHydrator->hydrate($shippingClass, new ProductShippingClass());
					}
				}

				return $hydrated;
			}
		));

		$hydrator->addStrategy('images', new ClosureStrategy(
			function() { return []; },
			function($images) use ($imageHydrator) {
				$hydrated = [];

				if (is_array($images)) {
					foreach ($images as $image) {
						$hydrated[] = $imageHydrator->hydrate($image, new ProductImage());
					}
				}

				return $hydrated;
			}
		));

		return $hydrator;
	}
}