<?php
namespace Ep\Gath\WordPress;

use Ep\Gath\Entity\Price;
use Ep\Gath\Exception;
use Ep\Gath\GathService;

/**
 * Overrida podatke o cijeni proizvoda s podacima iz Gatha.
 *
 * @package Ep\Gath\WordPress
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class PriceProvider extends ProductMetadataProvider
{
	/**
	 * Ključevi metapodataka koje ovaj provider overrida.
	 *
	 * @var array
	 */
	protected $metaKeys = [
		'_price',
		'_regular_price',
		'_regular_price_no_tax',
		'_sale_price',
		'_sale_price_dates_from',
		'_sale_price_dates_to',

		'_sale_percentage',
		'_rebate_percentage',
		'_net_price',
	];

	/**
	 * Učitava cijene i zalihu za date proizvode.
	 */
	public function loadMetadata(): void
	{
		try {
			if (count($this->preload) > 0) {
				$prices = $this->gath->getProductPrices(array_values($this->preload));

				foreach ($prices as $price) {
					if ($postId = array_search($price->getSku(), $this->preload)) {
						$this->loaded[$postId] = $price;
					} /*else {
						throw new Exception\MetadataException('Dobio sam podatke proizvoda koje nisam tražio.');
					}*/
				}
			}
		} catch (Exception\ExceptionInterface $e) {
			throw new Exception\MetadataException('Došlo je do nekakve greške s Gathom.', 0, $e);
		}
	}

	/**
	 * Vraća meta podatak za dati post s datim ključem.
	 *
	 * @param int $postId
	 * @param string $metaKey
	 * @param mixed|null $metaValue
	 *
	 * @return mixed
	 */
	public function getMetadata(int $postId, string $metaKey, $metaValue = null)
	{
		if ($postId > GathService::PRODUCT_ID_OFFSET) {
			$postId -= GathService::PRODUCT_ID_OFFSET;
		}

		if (!$this->getPrice($postId)) {
			$this->load();
		}

		$webshopSettings = get_option('ep_webshop', []);

		if (isset($webshopSettings['type']) && $webshopSettings['type'] === 'wholesale') {
			return $this->getMetadataWholesale($postId, $metaKey);
		}

		return $this->getMetadataRetail($postId, $metaKey);
	}

	/**
	 * Vraća metapodatke za cijene za veleprodaju.
	 *
	 * @param int $postId
	 * @param string $metaKey
	 *
	 * @return float|string
	 */
	public function getMetadataWholesale(int $postId, string $metaKey)
	{
		if (!in_array($metaKey, $this->metaKeys)) {
			return null;
		}

		$price = $this->getPrice($postId);

		if (!$price) {
			return '';
		}

		$productPrice = $price->hasRebate()
            ? $price->getRebate()->getDiscountedPrice()
            : $price->getPriceWholesale();

		if ($price->hasDiscount() && $price->getDiscount()->getDiscountedPriceNoTax() <= $productPrice) {
		    $productPrice = $price->getDiscount()->getOriginalPriceNoTax();
		    $regularPrice = $productPrice;
		    $salePrice = $price->getDiscount()->getDiscountedPriceNoTax();
		    $netPrice = $price->getDiscount()->getOriginalPriceNoTax();
            $rebatePercentage = $price->getDiscount()->getDisplayRate();
        } else {
		    $regularPrice = '';
		    $salePrice = '';
		    $netPrice = $price->hasRebate()
                ? $price->getRebate()->getOriginalPrice()
                : $price->getPriceWholesale();
		    $rebatePercentage = $price->hasRebate()
                ? $price->getRebate()->getRate()
                : 0;
        }

		switch ($metaKey) {
			case '_price':
                return $productPrice;
			case '_regular_price':
			    return $regularPrice;
			case '_sale_price':
                return $salePrice;
			case '_net_price':
				return $netPrice;
			case '_rebate_percentage':
				return $rebatePercentage;
		}

		return '';
	}

	/**
	 * Vraća metapodatke za cijene za maloprodaju.
	 *
	 * @param int $postId
	 * @param string $metaKey
	 *
	 * @return mixed
	 */
	public function getMetadataRetail(int $postId, string $metaKey)
	{
		if (!in_array($metaKey, $this->metaKeys)) {
			return null;
		}

		$price = $this->getPrice($postId);

		if (!$price) {
			return '';
		}

		switch ($metaKey) {
			case '_price':
				return $price->getPriceRetail();
			case '_regular_price':
				return $price->hasDiscount()
					? $price->getDiscount()->getOriginalPrice()
					: $price->getPriceRetail();
				break;
			case '_regular_price_no_tax':
				return $price->hasDiscount()
					? $price->getDiscount()->getOriginalPriceNoTax()
					: $price->getPriceWholesale();
			case '_sale_price':
				return $price->hasDiscount()
					? $price->getDiscount()->getDiscountedPrice()
					: '';
				break;
			case '_sale_percentage':
				return $price->hasDiscount()
					? $price->getDiscount()->getDisplayRate()
					: '';
		}

		return '';
	}

	/**
	 * Vraća cijenu i zalihe za dati proizvod.
	 *
	 * @param int $postId
	 *
	 * @return Price|null
	 */
	protected function getPrice(int $postId): ?Price
	{
		if (!isset($this->loaded[$postId])) {
			$this->preloadMetadata([$postId]);
		}

		return $this->loaded[$postId] ?? null;
	}
}