<?php
namespace Ep\Gath\WordPress;

use DateInterval;
use Ep\Core\UserProviderInterface;
use Ep\Gath\Entity\Product;
use Ep\Gath\Entity\ProductImage;
use Ep\Gath\GathService;
use WP_Post;
use WP_Query;
use WP_Term;
use Zend\Form\Element\DateTime;
use Zend\Stdlib\ArrayUtils;

/**
 * Class ProductProvider
 *
 * @package Ep\Gath\WordPress
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ProductProvider extends ProductMetadataProvider
{
	/** @var array */
	protected $metaKeys = [
		'_product_image_gallery',
		'_sku',
		'_tecdoc_key',
		'_tecDoc_key',
		'_tax_status',
		'_tax_class',
		'_thumbnail_id',
		'_weight',
	];

	/** @var Product[] */
	protected $products = [];

	/**
	 * ProductProvider constructor.
	 *
	 * @param GathService $gath
	 * @param UserProviderInterface $users
	 */
	public function __construct(GathService $gath, UserProviderInterface $users)
	{
		parent::__construct($gath, $users);

		add_filter('pre_get_posts', function(WP_Query $query) {
			$query->set('no_found_rows', false);
		});

		add_filter('posts_pre_query', [$this, 'getPosts'], 10, 2);
		add_filter('woocommerce_data_stores', [$this, 'dataStores'], 10, 1);
/*
		add_filter('woocommerce_is_purchasable', function() {
			return true;
		}, 99);
*/

		add_filter('woocommerce_product_subcategories_hide_empty', function() {
			return false;
		});

		add_filter('the_title', [$this, 'filterProductTitle'], 10, 2);

		add_filter('woocommerce_add_to_cart_validation', function($valid, $productId) {
			$this->getProduct($productId);
			return $valid;
		}, 10, 2);

		add_filter('wp_get_object_terms', [$this, 'getObjectTerms'], 10, 4);
		add_filter('get_the_terms', [$this, 'getTerms'], 10, 3);

		add_action('wp_ajax_flatsome_quickview', [$this, 'preloadPost'], 1);
		add_action('wp_ajax_nopriv_flatsome_quickview', [$this, 'preloadPost'], 1);
	}

    /**
     * @throws \Exception
     */
	public function preloadPost()
	{
		$productId = intval($_POST['product']);
		$this->getProduct($productId);
	}

    /**
     * @param $title
     * @param $productId
     *
     * @return string
     * @throws \Exception
     */
	public function filterProductTitle($title, $productId)
	{
		if (empty($title)) {
			if ($product = $this->getProduct($productId)) {
				return $product->post_title;
			}
		}

		return $title;
	}

	/**
	 * @param array $stores
	 *
	 * @return array
	 */
	public function dataStores(array $stores)
	{
		$stores['product'] = WooDataStore::class;
		return $stores;
	}

    /**
     * @param int $productId
     *
     * @return WP_Post|null
     * @throws \Exception
     */
	public function getProduct(int $productId)
	{
		if ($productId > GathService::PRODUCT_ID_OFFSET) {
			$productId -= GathService::PRODUCT_ID_OFFSET;
		}

		if (array_key_exists($productId, $this->products)) {
			$products = [$this->products[$productId]];
			$posts = $this->loadPosts($products);
			return $posts[0];
		} else {
			$products = $this->gath->getProductsByIds([$productId]);
			$posts = $this->loadPosts($products);

			if (count($posts) > 0) {
				return $posts[0];
			}
		}

		return null;
	}

	/**
	 * @param int $productId
	 *
	 * @return Product|null
	 */
	public function getGathProduct(int $productId)
	{
		if ($productId > GathService::PRODUCT_ID_OFFSET) {
			$productId -= GathService::PRODUCT_ID_OFFSET;
		}

		return isset($this->products[$productId])
			? $this->products[$productId]
			: null;
	}

    /**
     * @param array $terms
     * @param $objectIds
     * @param $taxonomies
     * @param $args
     *
     * @return array
     * @throws \Exception
     */
	public function getObjectTerms(array $terms, $objectIds, /** @noinspection PhpUnusedParameterInspection */ $taxonomies, $args)
	{
		if (!is_array($objectIds)) {
			$objectIds = [$objectIds];
		}

		$objectIds = array_map('absint', $objectIds);
		$termIds = [];

		foreach ($objectIds as $id) {
			$this->getProduct($id);

			if ($id > GathService::PRODUCT_ID_OFFSET) {
				$id -= GathService::PRODUCT_ID_OFFSET;
			}

			$product = isset($this->products[$id]) ? $this->products[$id] : false;

			if (in_array('product_brand', $args['taxonomy'])) {
				if ($product && $product->hasBrand()) {
					$termIds[] = $product->getBrandId();
				}
			}
		}

		if (count($termIds) > 0) {
			$extraTerms = get_terms([
				'include' => $termIds,
				'hide_empty' => false,
			]);

			if (isset($args['fields']) && 'ids' === $args['fields']) {
				$extraTerms = array_map(function(WP_Term $term) {
					return $term->term_id;
				}, $extraTerms);
			}

			$terms = ArrayUtils::merge($terms, $extraTerms);
		}

		return $terms;
	}

    /**
     * @param $terms
     * @param $postId
     * @param $taxonomy
     *
     * @return array|int|\WP_Error
     * @throws \Exception
     */
	public function getTerms($terms, $postId, $taxonomy)
	{
		if ($taxonomy === 'product_brand') {
			$this->getProduct($postId);

			if ($postId > GathService::PRODUCT_ID_OFFSET) {
				$postId -= GathService::PRODUCT_ID_OFFSET;
			}

			if (isset($this->products[$postId]) && $this->products[$postId]->hasBrand()) {
				$terms = get_terms([
					'taxonomy' => $taxonomy,
					'include' => [$this->products[$postId]->getBrandId()],
					'hide_empty' => false,
				]);
			}
		}

		return $terms;
	}

    /**
     * @param $null
     * @param WP_Query $query
     *
     * @return WP_Post[]
     * @throws \Exception
     */
	public function getPosts($null, WP_Query $query)
	{
        switch ($query->query_vars['orderby'] ?? '') {
            case 'price':
                $order = $query->query_vars['order'] === 'DESC'
                    ? GathService::SORT_PRICE_DESC
                    : GathService::SORT_PRICE_ASC;
                break;
            case 'date ID':
            default:
                $order = GathService::SORT_DATE_DESC;
                break;
        }

        if ($genericArticleIds = $query->get('moto_genids')) {
            $ids = array_map('absint', explode(';', $genericArticleIds));
            $motoId = intval($query->get('moto_id'));

            if (0 === $motoId) {
                return [];
            }

            $query->set('no_found_rows', false);
            $limit = $query->query_vars['posts_per_page'];
            $page = max(1, intval($query->query_vars['paged']));
            $offset = ($page - 1) * $limit;

            $products = $this->gath->getMotoProducts($motoId, $ids, $order, $limit, $offset);

            $query->found_posts = $this->gath->getFoundRows();
            $query->max_num_pages = ceil($query->found_posts / $limit);

            return $this->loadPosts($products);
        }

		foreach ($query->meta_query->queries as $metaQuery) {
			if (isset($metaQuery['key']) && in_array($metaQuery['key'], ['_tecdoc_key', '_tecDoc_key'])) {
				$tecDocPairs = [];

				foreach ($metaQuery['value'] as $key) {
					[$brandNo, $articleNo] = explode('|', $key);
					$tecDocPairs[] = [
						'BrandNo' => intval($brandNo),
						'ArticleNo' => $articleNo,
					];
				}

				$query->set('no_found_rows', false);
				$limit = $query->query_vars['posts_per_page'];
				$page = max(1, intval($query->query_vars['paged']));
				$offset = ($page - 1) * $limit;

				$products = $this->gath->getTecDocProducts($tecDocPairs, $order, $limit, $offset);

				$query->found_posts = $this->gath->getFoundRows();
				$query->max_num_pages = ceil($query->found_posts / $limit);

				return $this->loadPosts($products);
			}
		}

		if (
			'product' === $query->query_vars['post_type'] ||
			(is_array($query->query_vars['post_type']) && in_array('product', $query->query_vars['post_type']))
		) {
			if (count($query->query_vars['post__in']) > 0) {
				$productIds = [];

				foreach ($query->query_vars['post__in'] as $productId) {
					$productIds[] = $productId > GathService::PRODUCT_ID_OFFSET
						? $productId - GathService::PRODUCT_ID_OFFSET
						: $productId;
				}

				$products = $this->gath->getProductsByIds($productIds);
				return $this->loadPosts($products);
			}

			if (preg_match('/-(\d+)$/', $query->query_vars['name'], $matches)) {
				if ($post = $this->getProduct(intval($matches[1]))) {
					return [$post];
				}
			}
		} else if (!empty($query->query_vars['product_cat'])) {
			$query->set('no_found_rows', false);
			$limit = $query->query_vars['posts_per_page'];
			$page = max(1, intval($query->query_vars['paged']));
			$offset = ($page - 1) * $limit;

			$products = $this->gath->getProductsByCategory(get_queried_object_id(), $order, $limit, $offset);

			$query->found_posts = $this->gath->getFoundRows();
			$query->max_num_pages = ceil($query->found_posts / $limit);

			return $this->loadPosts($products);
		}

		return $null;
	}

    /**
     * Konvertira array proizvoda u array WP_Postova.
     *
     * @param Product[] $products
     *
     * @return WP_Post[]
     * @throws \Exception
     */
	protected function loadPosts(array $products)
	{
		$posts = [];

		foreach ($products as $product) {
			if (!is_object($product)) {
				continue;
			}

			if (!isset($this->products[$product->getId()])) {
				$this->products[$product->getId()] = $product;
			}

			$postId = GathService::PRODUCT_ID_OFFSET + $product->getId();
			$postDate = '2016-11-16 01:00:00';
			$postDateGmt = '2016-11-16 00:00:00';
			$postName = sprintf('%s-%d', sanitize_title($product->getName()), $product->getId());

			if ($product->getDateModified() instanceof DateTime) {
				$postModified    = $product->getDateModified()->format('Y-m-d H:i:s');
				$postModifiedGmt = $product->getDateModified()->sub(new DateInterval('PT3600S'))->format('Y-m-d H:i:s');
			} else {
				$postModified = $postDate;
				$postModifiedGmt = $postDateGmt;
			}

			$post = get_post(new WP_Post((object) [
				'ID' => $postId,
				'post_author' => 1,
				'post_date' => $postDate,
				'post_date_gmt' => $postDateGmt,
				'post_content' => '',
				'post_title' => $product->getName(),
				'post_excerpt' => $product->getShortDescription(),
				'post_status' => 'publish',
				'comment_status' => 'closed',
				'ping_status' => 'closed',
				'post_password' => '',
				'post_name' => $postName,
				'to_ping' => '',
				'pinged' => '',
				'post_modified' => $postModified,
				'post_modified_gmt' => $postModifiedGmt,
				'post_content_filtered' => '',
				'post_parent' => 0,
				'guid' => sprintf('%s?p=%d', get_option('siteurl'), $postId),
				'post_type' => 'product',

				'filter' => 'raw',
			]));

			wp_cache_add($post->ID, $post, 'posts');
			$posts[] = $post;
		}

		return $posts;
	}

	/**
	 * Vraća vrijednost metapodatka iz nekog drugog izvora, ovisno o implementaciji.
	 *
	 * @param int $postId
	 * @param string $metaKey
	 *
	 * @param null $metaValue
	 *
	 * @return mixed
	 */
	public function getMetadata(int $postId, string $metaKey, $metaValue = null)
	{
		if ($postId > GathService::PRODUCT_ID_OFFSET) {
			$postId = $postId - GathService::PRODUCT_ID_OFFSET;
		}

		if (!isset($this->products[$postId])) {
			return '';
		}

		$product = $this->products[$postId];

		switch ($metaKey) {
			case '_product_image_gallery':
				if ($product->hasImages() && count($product->getImages()) > 1) {
					$images = array_map(function(ProductImage $image) {
						return sprintf('gath-%s', $image->getPath());
					}, $product->getImages());
					array_shift($images);
					return implode(',', $images);
				}
				break;
			case '_sku':
				return $product->getSku();
			case '_tecdoc_key':
			case '_tecDoc_key':
				return sprintf(
					'%s|%s',
					$product->getTecdocBrandNo(),
					$product->getTecdocArticleNo()
				);
			case '_tax_status':
				return 'taxable';
			case '_tax_class':
				return $product->getTaxClass();
			case '_thumbnail_id':
				return $product->hasThumbnail()
					? sprintf('gath-%s', $product->getThumbnail()->getPath())
					: '';
			case '_weight':
				return 0;
		}

		return '';
	}

	/**
	 * Učitava metapodatke za proizvode u $this->load.
	 */
	function loadMetadata()
	{ }
}